Utilities
=========

General Infrastructure
----------------------

.. toctree::
   :maxdepth: 1

   sage/misc/defaults
   sage/misc/functional
   sage/misc/randstate
   sage/misc/prandom
   sage/misc/unknown

Programming Utilities
---------------------

Special Base Classes, Decorators, etc.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/abstract_method
   sage/misc/bindable_class
   sage/misc/decorators
   sage/misc/constant_function
   sage/misc/classcall_metaclass
   sage/misc/inherit_comparison
   sage/misc/method_decorator
   sage/misc/object_multiplexer
   sage/misc/fast_methods
   sage/misc/call

Lists and Iteration, etc.
~~~~~~~~~~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/callable_dict
   sage/misc/converting_dict
   sage/misc/flatten
   sage/misc/search
   sage/misc/mrange
   sage/misc/multireplace
   sage/misc/map_threaded
   sage/arith/srange
   sage/misc/element_with_label

File and OS Access
~~~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/temporary_file
   sage/misc/remote_file
   sage/misc/messaging
   sage/misc/sage_ostools

Database Access
~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/databases/sql_db

Warnings
~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/stopgap
   sage/misc/superseded


Miscellaneous Useful Functions
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/misc
   sage/misc/misc_c
   sage/misc/verbose

Laziness
~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/lazy_attribute
   sage/misc/lazy_format
   sage/misc/lazy_import
   sage/misc/lazy_import_cache
   sage/misc/lazy_list
   sage/misc/lazy_string

Caching
~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/cachefunc
   sage/misc/weak_dict

Fast Expression Evaluation
~~~~~~~~~~~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/ext/fast_callable
   sage/ext/fast_eval

.. The docbuilder only accepts modules from src/sage/* in the reference
.. manual (#18216)
..   sage_setup/autogen/interpreters

.. Automatically generated interpreters, probably shouldn't be documented.
..   sage/ext/interpreters/wrapper_cdf
..   sage/ext/interpreters/wrapper_el
..   sage/ext/interpreters/wrapper_py
..   sage/ext/interpreters/wrapper_rdf
..   sage/ext/interpreters/wrapper_rr

Code Evaluation
---------------

.. toctree::
   :maxdepth: 1

   sage/misc/cython
   sage/misc/inline_fortran
   sage/misc/parser
   sage/misc/python
   sage/misc/sage_eval
   sage/misc/sh

Formatted Output
----------------

Formatted Output
~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/typeset/symbols
   sage/typeset/character_art
   sage/typeset/character_art_factory
   sage/typeset/ascii_art
   sage/typeset/unicode_art
   sage/typeset/unicode_characters
   sage/misc/repr
   sage/misc/sage_input
   sage/misc/table

HTML and MathML
~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/html
   sage/misc/mathml

LaTeX
~~~~~

.. toctree::
   :maxdepth: 1

   sagetex
   sage/misc/latex
   sage/misc/latex_macros
   sage/misc/latex_standalone

Saving and Loading Sage Objects
-------------------------------

.. toctree::
   :maxdepth: 1

   sage/misc/func_persist
   sage/misc/persist
   sage/misc/fpickle
   sage/misc/explain_pickle
   sage/misc/nested_class
   sage/misc/session

Interactive Usage Support
-------------------------

Interactive Sage Sessions
~~~~~~~~~~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/banner
   sage/misc/reset
   sage/misc/viewer
   sage/misc/pager
   sage/misc/sagedoc
   sage/misc/sagedoc_conf
   sage/misc/sphinxify

Distribution
~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/package


Credits
~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/citation
   sage/misc/copying

Development Tools
-----------------

Testing
~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/sage_unittest
   sage/misc/random_testing
   sage/misc/test_nested_class

Benchmarking and Profiling
~~~~~~~~~~~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/benchmark
   sage/misc/sage_timeit
   sage/misc/sage_timeit_class
   sage/misc/profiler
   sage/misc/gperftools

Miscellaneous Inspection and Development Tools
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

.. toctree::
   :maxdepth: 1

   sage/misc/instancedoc
   sage/misc/sageinspect
   sage/misc/edit_module
   sage/misc/classgraph
   sage/misc/dev_tools
   sage/misc/function_mangling
   sage/misc/rest_index_of_methods

Low-Level Utilities
-------------------

.. toctree::
   :maxdepth: 1

   sage/ext/memory
   sage/misc/c3
   sage/misc/c3_controlled

.. include:: ../footer.txt
